# frozen_string_literal: true

require 'cht-chaphql'
require 'cht-chaphql/graphql_formatters'

SUBSCRIPTION_FMTRS =
  YAML.load_file(File.join(__dir__, '../helpers/formatters.yml'))

# module for subscription
module ChtApitestSubscription
  include Chaphql
  include Chaphql::GraphQLHelpers

  # module for your shortcuts
  module SubscriptionHelpers
    # insertion point for shortcuts

    def create_subscription(args = {})
      args[:service] = SubscriptionService.new
      sym = first_no_cap(camelize('create_subscription')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = { input: { endDate: args[:endDate], name: args[:name], recipients: { email: args[:email], s3Destination: { accountId: args[:accountId], s3Bucket: args[:s3Bucket] } }, schedule: { executionDays: args[:executionDays], executionTimeOfDay: args[:executionTimeOfDay], scheduleFrequency: args[:scheduleFrequency] }, startDate: args[:startDate], target: args[:target] } }
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def update_subscription(args = {})
      args[:service] = SubscriptionService.new
      sym = first_no_cap(camelize('update_subscription')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = { id: args[:id], input: { endDate: args[:endDate], name: args[:name], recipients: { email: args[:email], s3Destination: { accountId: args[:accountId], s3Bucket: args[:s3Bucket] } }, schedule: { executionDays: args[:executionDays], executionTimeOfDay: args[:executionTimeOfDay], scheduleFrequency: args[:scheduleFrequency] }, startDate: args[:startDate], target: args[:target] } }
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def delete(args = {})
      args[:service] = SubscriptionService.new
      sym = first_no_cap(camelize('delete')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = { id: args[:id] }
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def node(args = {})
      args[:service] = SubscriptionService.new
      sym = first_no_cap(camelize('node')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = 
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def subscriptions(args = {})
      args[:service] = SubscriptionService.new
      sym = first_no_cap(camelize('subscriptions')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = { input: { name: args[:name], target: args[:target], type: args[:type] } }
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def subscription_metadata(args = {})
      args[:service] = SubscriptionService.new
      sym = first_no_cap(camelize('subscription_metadata')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = {  }
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    # end of shortcut insertion
  end

  # class for your service
  class SubscriptionService < ChtNgService
    include RSpec::Matchers

    SUBSCRIPTION_DEFAULTS = {
      name: 'Subscription',
      default_namespace: 'subscription',
      service_name: 'subscription',
      pr_parent: 'subscription',
      formatters: SUBSCRIPTION_FMTRS['formatters'],
      checkers: SUBSCRIPTION_FMTRS['checkers']
    }.freeze
    def initialize(args = {})
      super(args.merge(SUBSCRIPTION_DEFAULTS))
    end
  end

  # class for api-proxy support of subscription service
  class ApiProxySubscription < ChtNgService
    include RSpec::Matchers

    API_PROXY_DEFAULTS = {
      name: 'ApiProxySubscription',
      default_namespace: 'api-proxy',
      service_name: 'api-proxy',
      pr_parent: 'api-proxy',
      formatters: SUBSCRIPTION_FMTRS['formatters'],
      checkers: SUBSCRIPTION_FMTRS['checkers']
    }.freeze
    def initialize(args = {})
      super(args.merge(API_PROXY_DEFAULTS))
    end
  end
end
