require 'spec_helper'

module SubscriptionTest
  describe 'Subscription createSubscription' do
    include_context 'request metadata'

    def validate_create_or_update_subscription_result(json_body, name, frequency, target, start_date, end_date, emails = nil, s3_bucket = nil, account = nil)
      failures = []
      failures << "#{json_body[:data][:createSubscription][:name]}" unless expect(json_body[:data][:createSubscription][:name]).to eq(name)
      failures << "#{json_body[:data][:createSubscription][:schedule][:scheduleFrequency]}" unless expect(json_body[:data][:createSubscription][:schedule][:scheduleFrequency]).to eq(frequency)
      failures << "#{json_body[:data][:createSubscription][:target]}" unless expect(json_body[:data][:createSubscription][:target]).to eq(target)
      failures << "#{json_body[:data][:createSubscription][:startDate]}" unless expect(json_body[:data][:createSubscription][:startDate]).to eq(start_date,)
      failures << "#{json_body[:data][:createSubscription][:endDate]}" unless expect(json_body[:data][:createSubscription][:endDate]).to eq(end_date)
      failures << "#{json_body[:data][:createSubscription]}" unless expect(json_body[:data][:createSubscription][:recipients][:s3Destination][:s3Bucket]).to eq(s3_bucket) if !s3_bucket.nil?
      failures << "#{json_body[:data][:createSubscription]}" unless expect(json_body[:data][:createSubscription][:recipients][:s3Destination][:accountId]).to eq(account) if !account.nil?
      failures
    end

    before(:all) do
      @target = 'crn:2017:flexreports/d062a54e-8f09-4885-b04a-3da2f32b7059'
      @target = 'crn:57534:flexreports/1118dfdb-0b6f-4cae-acb4-90a675de16bd' if ENV['TEST_ENV'].eql? 'preprod'
      @start_date = Date.today
      @end_date = (@start_date + 30) # One Month
      @end_date_monthly = (@start_date + 90)
      @emails = ['satyanarayaj@cloudhealthtech.com']
      @s3_bucket = 'satyanarayaj-fr-test',
      @subscription_name = 'Rspec Subscription Report Automation',
      @daily_execution_days = ['7']
      @monthly_execution_days = ['27','29']
      @first_and_last_day_of_month = ['1', '-1']
      @account = '3367254377911'
    end

    it 'can create DAILY subscription for a Report' do
      failures = []
      wrap_request do
        variables = {
          'input': {
            'name': @subscription_name,
            'endDate': @end_date.to_s,
            'startDate': @start_date.to_s,
            'target': @target,
            'recipients': {
              'email': @emails
            },
            'schedule': {
              'executionDays': @daily_execution_days,
              'executionTimeOfDay': '23:25:59+05:30',
              'scheduleFrequency': 'DAILY'
              }
            }
          }
        graphql url(service: service),
                format_request(
                  :createSubscription,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        expect(validate_create_or_update_subscription_result(json_body, @subscription_name, 'DAILY', @target, @start_date.to_s, @end_date.to_s, @emails)).to eq([])
      end
    end

    it 'can create MONTHLY subscription for a Report' do
      failures = []
      wrap_request do
        variables = {
          'input': {
            'name': @subscription_name,
            'endDate': @end_date_monthly.to_s,
            'startDate': @start_date.to_s,
            'target': @target,
            'recipients': {
              'email': @emails
            },
            'schedule': {
              'executionDays': @monthly_execution_days,
              'executionTimeOfDay': '23:25:59+05:30',
              'scheduleFrequency': 'MONTHLY'
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :createSubscription,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        expect(validate_create_or_update_subscription_result(json_body, @subscription_name, 'MONTHLY', @target, @start_date.to_s, @end_date_monthly.to_s, @emails)).to eq([])
      end
    end

    it 'can create FIRST and LAST Day of the month as subscription for a Report' do
      failures = []
      wrap_request do
        variables = {
          'input': {
            'name': @subscription_name,
            'endDate': @end_date_monthly.to_s,
            'startDate': @start_date.to_s,
            'target': @target,
            'recipients': {
              'email': @emails
            },
            'schedule': {
              'executionDays': @first_and_last_day_of_month,
              'executionTimeOfDay': '23:25:59+05:30',
              'scheduleFrequency': 'MONTHLY'
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :createSubscription,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        expect(validate_create_or_update_subscription_result(json_body, @subscription_name, 'MONTHLY', @target, @start_date.to_s, @end_date_monthly.to_s, @emails)).to eq([])
      end
    end

    # Staging is not working with creating s3 delivery so commenting this test for now
    # TODO: https://cloudhealthtech.atlassian.net/browse/ZEUS-2625
    xit 'can create s3 bucket as delivery option for the subscription of report' do
      failures = []
      wrap_request do
        variables = {
          'input': {
            'name': @subscription_name,
            'endDate': @end_date_monthly.to_s,
            'startDate': @start_date.to_s,
            'target': @target,
            'recipients': {
              's3Destination': {
              's3Bucket': @s3_bucket,
              'accountId': @account
              }
            },
            'schedule': {
              'executionDays': @monthly_execution_days,
              'executionTimeOfDay': '23:25:59+05:30',
              'scheduleFrequency': 'MONTHLY'
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :createSubscription,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        expect(validate_create_or_update_subscription_result(json_body, @subscription_name, 'MONTHLY', @target, @start_date.to_s, @end_date_monthly.to_s, @emails, @s3_bucket, @account)).to eq([])
      end
    end

    it 'can create subscription with no end date provided' do
      failures = []
      wrap_request do
        variables = {
          'input': {
            'name': @subscription_name,
            'startDate': @start_date.to_s,
            'target': @target,
            'recipients': {
              'email': @emails
            },
            'schedule': {
              'executionDays': @monthly_execution_days,
              'executionTimeOfDay': '23:25:59+05:30',
              'scheduleFrequency': 'MONTHLY'
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :createSubscription,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        expect(validate_create_or_update_subscription_result(json_body, @subscription_name, 'MONTHLY', @target, @start_date.to_s, nil, @emails)).to eq([])
      end
    end
  end
end
