require 'spec_helper'

module SubscriptionTest
  describe 'List and Delete Subscriptions' do
    include_context 'request metadata'
    before(:all) do
      @ids = []
      @target = 'crn:2017:flexreports/d062a54e-8f09-4885-b04a-3da2f32b7059'
      @target = 'crn:57534:flexreports/1118dfdb-0b6f-4cae-acb4-90a675de16bd' if ENV['TEST_ENV'].eql? 'preprod'
      @start_date = Date.today
      @end_date = (@start_date + 30) # One Month
      @end_date_monthly = (@start_date + 90)
      @emails = ['satyanarayaj@cloudhealthtech.com']
      @s3_bucket = 'satyanarayaj-fr-test',
        @subscription_name = 'Rspec Subscription Report Automation',
        @daily_execution_days = ['7']
      @monthly_execution_days = ['27','29']
      @first_and_last_day_of_month = ['1', '-1']
      @account = '3367254377911'
    end

    it 'can list all available ng subscriptions matching a specific name | type | target' do
      failures = []
      wrap_request do
        variables = {
          'input': {
            'name': @subscription_name
          }
        }
        graphql url(service: service),
                format_request(
                  :subscriptions,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        failures << 'Subscriptions List Response with input as traget returns empty list' if json_body[:data][:subscriptions].nil?
        name = json_body[:data][:subscriptions].map { |subscription| subscription[:name] }.uniq
        failures << 'Automation Subscription has different names other than specified name' unless name.first.eql?(@subscription_name)
        @ids = json_body[:data][:subscriptions].map { |subscription| subscription[:id] }

        wrap_request do
          variables = {
            'input': {
              'target': @target
            }
          }
          graphql url(service: service),
                  format_request(
                    :subscriptions,
                    service: service
                  ),
                  variables, headers
          expect_success_graphql
          failures << 'Subscriptions List Response with input as traget returns empty list' if json_body[:data][:subscriptions].nil?
          targets_actual = json_body[:data][:subscriptions].map { |subscription| subscription[:target] }.uniq
          failures << 'Subscriptions other than the Matching target are fetched' unless targets_actual.first.eql? @target
          @ids = json_body[:data][:subscriptions].map { |subscription| subscription[:id] }

          wrap_request do
            variables = {
              'input': {
                'type': 'flexreports'
              }
            }
            graphql url(service: service),
                    format_request(
                      :subscriptions,
                      service: service
                    ),
                    variables, headers
            expect_success_graphql
            failures << 'Subscriptions List Response with input as traget returns empty list' if json_body[:data][:subscriptions].nil?
            crn = json_body[:data][:subscriptions].map { |subscription| subscription[:target].split(':')[2].split('/')[0] }.uniq
            failures << 'Subscriptions other than the Matching type are fetched' unless crn.first.eql? 'flexreports'
          end
        end
      end
      expect(failures).to eq([])
    end

      skip("No Matching Subscriptions found to delete") if @ids.nil?
    it 'can delete above listed subscriptions' do
      @ids.each do |id|
        wrap_request do
          variables = {
            'id': id
          }
          graphql url(service: service),
                  format_request(
                    :delete,
                    service: service
                  ),
                  variables, headers
          expect_success_graphql
          check(:delete, service: service)
        end
      end
    end
  end
end

