require 'spec_helper'

module SubscriptionTest
  describe 'Subscription updateSubscription' do
    include_context 'request metadata'

    before(:all) do
      @subscription_id = 'crn:2017:subscription/4c1f3bd0-3bbe-4260-98d0-3de1965d7ad5'
      @subscription_id = 'crn:57534:subscription/98c44dea-25be-4706-9c7a-f9a040a8dee5' if ENV['TEST_ENV'].eql? 'preprod'
      @subscription_name = 'DONOT DELETE NODE API REPORT RSPEC AUTOMATION'
      @monthly_execution_days = [7,14,21,28]
      @daily_execution_days = [2,4,6]
      @daily_email = ['satyanarayaj@cloudhealthtech.com']
      @monthly_email = ['kapilku@cloudhealthtech.com']
      @failures = []
    end

    it 'can update an existing Subscription FROM DAILY to MONTHLY or MONTHLY to DAILY' do
      # fetch subscription details before updating
      wrap_request do
        variables = {}
        graphql url(service: service),
                format_request(
                  :node,
                  service: service,
                  crn: @subscription_id,
                  type: 'CHSubscription',
                  fields: 'id name target startDate endDate status lastInvocationResult schedule { scheduleFrequency executionDays executionTimeOfDay }  recipients { email }'
                ),
                variables, headers
        expect_success_graphql
        check(:node, service: service)
        @failures << "Not able to get subscription details for subscription #{@subscription_id}" if json_body[:data][:node].nil?
        expect(@failures).to eq([])
        @execution_days = json_body[:data][:node][:schedule][:executionDays]
        @execution_frequency = json_body[:data][:node][:schedule][:scheduleFrequency]
        @name = json_body[:data][:node][:schedule][:name]
        @start_date = json_body[:data][:node][:startDate]
        @end_date = json_body[:data][:node][:endDate]
        @target = json_body[:data][:node][:target]
      end

      if @execution_frequency.eql? 'MONTHLY'
        execution_days = @daily_execution_days
        frequency = 'DAILY'
        email = @daily_email
      elsif @execution_frequency.eql? 'DAILY'
        frequency = 'MONTHLY'
        execution_days = @monthly_execution_days
        email = @monthly_email
      else
        raise('UNKNOWN FREQUNCY IN SCHEDULE')
      end
      wrap_request do
        variables = {
          'id': @subscription_id,
          'input': {
            'endDate': @end_date,
            'name': @subscription_name,
            'recipients': {
              'email': email
            },
            'schedule': {
              'executionDays': execution_days,
              'executionTimeOfDay': '23:25:59+05:45',
              'scheduleFrequency': frequency
            },
            'startDate': @start_date,
            'target': @target
          }
        }
        graphql url(service: service),
                format_request(
                  :updateSubscription,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        @failures << "Not able to Update subscriptipon #{@subscription_id}" if json_body[:data][:updateSubscription].nil?
        @failures << 'scheduleFrequency is not correctly updated' unless json_body[:data][:updateSubscription][:schedule][:scheduleFrequency].eql? frequency
        @failures << 'Name is not correct for the updated subscription' unless json_body[:data][:updateSubscription][:name].eql? @subscription_name
        @failures << 'start date value is incorrect' unless json_body[:data][:updateSubscription][:startDate].eql? @start_date
        @failures << 'end date value is incorrect' unless json_body[:data][:updateSubscription][:endDate].eql? @end_date
        @failures << 'executionDays are wrongly update' unless json_body[:data][:updateSubscription][:schedule][:executionDays].eql? execution_days
      end
      expect(@failures).to eq([])
    end
  end
end
